/*=============================================================================

				    cutnmr.c

			cut pdb file with nmr coordinates

Purpose: extract only one model from pdb file (nmr data usualy contain many)

History:
(1)	 6.3.1998	created

========includes:============================================================*/

#include <stdio.h>
#include <string.h>

/*======defines:=============================================================*/

/* No defines */

/*======function prototypes:=================================================*/

void	PrintUsage (void);
int	ExtractModel (char *, char *);
int	RenameFile (char *, char *, int);
void	ErrorMessage (char *, char *, char *, char *, char *, char *, char *);
void	WarningMessage (char *, char *, char *,
			char *, char *, char *, char *);

/*======global scope data:===================================================*/

/* No global scope data */

/*======main:================================================================*/

int main (int argc, char *argv[])
{
int		n1, n2;
char		InFileName[1000], OutFileName[1000];
/* Print usage info if number of command line arguments is bad: */
if (argc != 3)
	{
	PrintUsage ();
	exit (0);
	}

/* Prepare the file names: */
n1 = sizeof (InFileName) - 1;
strncpy (InFileName, argv[1], n1);
InFileName[n1] = '\0';
n2 = sizeof (OutFileName) - 1;
strncpy (OutFileName, argv[2], n2);
OutFileName[n2] = '\0';
/** Compare file names (should be different): **/
if (strcmp (InFileName, OutFileName) == 0)
	{
	ErrorMessage ("cutnmr", "main", InFileName,
		"The same file name is specified",
		" for both input and output file.\n", "", "");
	return -1;
	}

/* Do the real job: */
if (ExtractModel (InFileName, OutFileName) == -1) exit (0);

/* The end: */
return 1;
}

/*======usage info:==========================================================*/

void PrintUsage (void)
{
fprintf (stdout, "\n===========================================\n");
fprintf (stdout, "Remove unnecessary NMR models from pdb file\n");
fprintf (stdout, "===========================================\n\n");
fprintf (stdout, "Usage:                cutnmr infile outfile\n\n");
}

/*======extract one model from pdb file:=====================================*/

int ExtractModel (char *InFileName, char *OutFileName)
{
FILE		*InFile = NULL, *OutFile = NULL;
char		BackupFileName[1000];
int		MaxChars;
int		n;
char		Line[1000], ModelID[] = "MODEL";
int		ModelsN = 0;

/* Check input file: */
if ((InFile = fopen (InFileName, "r")) == NULL)
	{
	ErrorMessage ("cutnmr", "ExtractModel", InFileName,
			"Failed to open the input file.",
			" Check the specified pdb file!\n", "", "");
	return -1;
	}

/* Check output file: */
if ((OutFile = fopen (OutFileName, "r")) != NULL)
	{
	fclose (OutFile);
	OutFile = NULL;
	MaxChars = (int) sizeof (BackupFileName);
	n = RenameFile (BackupFileName, OutFileName, MaxChars);
	if (n > 0) WarningMessage ("cutnmr", "ExtractModel", OutFileName,
                        "The specified output file exists. It will be",
                        " renamed to ", BackupFileName, "\n");
	} 

/* Open the output file: */
if ((OutFile = fopen (OutFileName, "w")) == NULL)
	{
	ErrorMessage ("cutnmr", "ExtractModel", OutFileName,
			"Failed to open the output file.",
			" Check who is the owner of this file;\n",
			"check directory, protection",
			" and available disk space.\n"); 
	/** Don't forget to close input file: **/
	fclose (InFile);
	return -1;
	}

/* Line length: */
MaxChars = sizeof (Line);

/* Read all lines, break from the loop when one complete model is copied: */
while (fgets (Line, MaxChars, InFile) != NULL)
	{
	if (strstr (Line, ModelID) == Line)
		{
		if (++ModelsN >= 2) break;
		}
	fputs (Line, OutFile);
	}

/* Close files: */
if (InFile) fclose (InFile);
if (OutFile) fclose (OutFile);

return 1;
}

/*======rename file (put # before the old file name):========================*/

int RenameFile (char *BackupName, char *OldName, int FileNameLen)
{
int		SpaceLeft;

strcpy (BackupName, "#");
SpaceLeft = FileNameLen - strlen (BackupName) - 1;
if (SpaceLeft <= 0) return -1;
strncat (BackupName, OldName, SpaceLeft);
if (rename (OldName, BackupName) != 0) return -1;

return 1;
}

/*======print error message:=================================================*/

/* Note: there is no  newline  in the last  four fprintf! */
/* You are expected to supply them through the arguments! */

void ErrorMessage (char *module, char *function, char *filename,
		   char *s1, char *s2, char *s3, char *s4)
{
fprintf (stderr, "\nERROR>");
fprintf (stderr, " module: %s, function: %s", module, function);
if (strlen (filename) != 0) fprintf (stderr, ", file: %s", filename);
fprintf (stderr, "\n");
if (strlen (s1) != 0) fprintf (stderr, "%s", s1);
if (strlen (s2) != 0) fprintf (stderr, "%s", s2);
if (strlen (s3) != 0) fprintf (stderr, "%s", s3);
if (strlen (s4) != 0) fprintf (stderr, "%s", s4);
}

/*======print warning message:===============================================*/

/* Note: there is no  newline  in the last  four fprintf! */
/* You are expected to supply them through the arguments! */

void WarningMessage (char *module, char *function, char *filename,
		     char *s1, char *s2, char *s3, char *s4)
{
fprintf (stdout, "WARNING>");
fprintf (stdout, " module: %s, function: %s", module, function);
if (strlen (filename) != 0) fprintf (stdout, ", file: %s", filename);
fprintf (stdout, "\n");
if (strlen (s1) != 0) fprintf (stdout, "%s", s1);
if (strlen (s2) != 0) fprintf (stdout, "%s", s2);
if (strlen (s3) != 0) fprintf (stdout, "%s", s3);
if (strlen (s4) != 0) fprintf (stdout, "%s", s4);
}

/*===========================================================================*/


