
/*======includes:============================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "protein.h"
#include "pdb.h"

/*======function prototypes:=================================================*/

void		ErrorMessage_ (char *, char *, char *,
			       char *, char *, char *, char *);
int		ExtractNCACO_ (VectorS *, VectorS *, VectorS *, VectorS *,
				 long, long, AtomDataS *);
double		CalcPhi_ (VectorS *, VectorS *, VectorS *, VectorS *);
double		CalcPsi_ (VectorS *, VectorS *, VectorS *, VectorS *);

/*======calculate dihedral angles:===========================================*/

DihedS *CalcDihedralAngles_ (long *DihedsNP, ResidueS *RSP, long ResiduesN, 
			     AtomDataS *ASP)
{
DihedS		*DSP;
size_t		MemSize, DihedStructSize, VectorSize;
long		ResI;
VectorS		N, CA, C, O, COld;
ResidueS	*p;
int		n;
double		phi, psi;

/* Sizes of structures: */
DihedStructSize = sizeof (DihedS);
VectorSize = sizeof (VectorS);

/* Allocate the memory for dihedral angles (including reasonable margin): */
MemSize = DihedStructSize * (size_t) (ResiduesN + 100);
if (!(DSP = (DihedS *) malloc (MemSize)))
	{
	ErrorMessage_ ("dihed", "CalcDihedralAngles", "",
		"Failed to allocate memory.",
		" Check the amount of free memory!\n", "", "");
	return NULL;
	}

/* Initialize counter: */
*DihedsNP = 0L;

/* Extract the necessary atomic vectors for the first residue: */
n = ExtractNCACO_ (&N, &CA, &C, &O, RSP->Start, RSP->End, ASP);

/* Calculate angles for the first  residue, */
/*  or set error indicator if missing data: */
if (n == 4) psi = CalcPsi_ (&N, &CA, &C, &O);
else psi = ERRORINDICATOR;
DSP->psi = psi;
/** Set the phi value for the first residue (not calculated): **/
DSP->phi = ERRORINDICATOR;

/* Increase counter: */
if (psi != ERRORINDICATOR) (*DihedsNP)++;

/* Copy the position of C atom: */
memcpy (&COld, &C, VectorSize);

/* Process all residues except the first one: */
for (ResI = 1L; ResI < ResiduesN; ResI++)
	{
	/* Copy pointer: */
	p = RSP + ResI;

	/* Prepare atomic vectors: */
	n = ExtractNCACO_ (&N, &CA, &C, &O, p->Start, p->End, ASP);

	/* Calculate angles, or set error indicator if missing data: */
	if (n == 4)
		{
		phi = CalcPhi_ (&COld, &N, &CA, &C);
		psi = CalcPsi_ (&N, &CA, &C, &O);
		}
	else
		{
		phi = ERRORINDICATOR;
		psi = ERRORINDICATOR;
		}
	(DSP + ResI)->phi = phi;
	(DSP + ResI)->psi = psi;

	/* Increase counter: */
	if (phi != ERRORINDICATOR) (*DihedsNP)++;
	if (psi != ERRORINDICATOR) (*DihedsNP)++;

	/* Copy the old C atom position: */
	memcpy (&COld, &C, VectorSize);
	}

return DSP;
}

/*===========================================================================*/


