
/*======includes:============================================================*/

#include <stdio.h>
#include <math.h>
#include "common.h"

/*======function prototypes:=================================================*/

int		CheckBondLengths1_ (VectorS *, VectorS *, VectorS *);
int		VectorProduct_ (VectorS *, VectorS *, VectorS *);
double		AbsValue_ (VectorS *);
double		ScalarProduct_ (VectorS *, VectorS *);

/*======calculate the phi angle:=============================================*/

double CalcPhi_ (VectorS *COldP, VectorS *NP, VectorS *CAP, VectorS *CP)
{
VectorS		COldN, NCA, CAC, u1, u2, v1, v2;
static long	BadBondsN;
double		denom, ratio;
double		alpha, phi;

/* Prepare the three initial vectors: */
COldN.x = NP->x - COldP->x;
COldN.y = NP->y - COldP->y;
COldN.z = NP->z - COldP->z;
NCA.x = CAP->x - NP->x;
NCA.y = CAP->y - NP->y;
NCA.z = CAP->z - NP->z;
CAC.x = CP->x - CAP->x;
CAC.y = CP->y - CAP->y;
CAC.z = CP->z - CAP->z;

/* Check bond lengths: */
if (CheckBondLengths1_ (&COldN, &NCA, &CAC) < 0)
	{
	BadBondsN++;
	if (BadBondsN <= MAXMSG)
		{
		printf ("Chain cleaved, another subunit reached");
		printf (" or bad bond found ...\n");
		}
	else if (BadBondsN == MAXMSG + 1)
		{
		printf ("Bad bond (or cleavage) found again,");
		printf (" this is the last warning ...\n");
		}
 	return ERRORINDICATOR;
	}

/* Two vectors orthogonal on NCA, mutually */
/* orthogonal, one in the COldN-NCA plane: */
VectorProduct_ (&u1, &COldN, &NCA);
VectorProduct_ (&u2, &u1, &NCA);

/* Two vectors orthogonal on NCA, mutually */
/* orthogonal, one in the  CAC-NCA  plane: */
VectorProduct_ (&v1, &NCA, &CAC);
VectorProduct_ (&v2, &v1, &NCA);

/* The angle alpha, which will be used to calculate phi: */
denom = AbsValue_ (&v2) * AbsValue_ (&u2);
/** Avoid division by zero: **/
if (denom == 0.0)
	{
	printf ("Zero length vector found ...\n");
	return ERRORINDICATOR;
	}
ratio = ScalarProduct_ (&v2, &u2) / denom;
/** cos range is between -1 and 1: **/
if ((ratio > 1.0) || (ratio < -1.0)) return ERRORINDICATOR;
alpha = acos (ratio);

/* There are two possible solutions; the right one is resolved here: */
if (ScalarProduct_ (&v2, &u1) >= 0) phi = alpha;
else phi = -alpha;

/* Return the angle (in radians): */
return phi;
}

/*===========================================================================*/


