
/*======includes:============================================================*/

#include <stdio.h>
#include <math.h>
#include "common.h"

/*======function prototypes:=================================================*/

int		CheckBondLengths2_ (VectorS *, VectorS *, VectorS *);
int		VectorProduct_ (VectorS *, VectorS *, VectorS *);
double		AbsValue_ (VectorS *);
double		ScalarProduct_ (VectorS *, VectorS *);

/*======calculate the psi angle:=============================================*/

double CalcPsi_ (VectorS *NP, VectorS *CAP, VectorS *CP, VectorS *OP)
{
VectorS		CAN, CAC, CO, u1, u2, v1, v2;
static int	BadBondsN;
double		denom, ratio;
double		alpha, psi;


/* Prepare the three initial vectors: */
CAN.x = NP->x - CAP->x;
CAN.y = NP->y - CAP->y;
CAN.z = NP->z - CAP->z;
CAC.x = CP->x - CAP->x;
CAC.y = CP->y - CAP->y;
CAC.z = CP->z - CAP->z;
CO.x = OP->x - CP->x;
CO.y = OP->y - CP->y;
CO.z = OP->z - CP->z;

/* Check bond lengths: */
if (CheckBondLengths2_ (&CAN, &CAC, &CO) < 0)
	{
	BadBondsN++;
	if (BadBondsN <= MAXMSG)
		{
		printf ("Chain cleaved, another subunit reached");
		printf (" or bad bond found ...\n");
		}
	else if (BadBondsN == MAXMSG + 1)
		{
		printf ("Bad bond (or cleavage) found again,");
		printf (" this is the last warning ...\n");
		}
	return ERRORINDICATOR;
	}

/* Two vectors orthogonal on CAC, mutually */
/* orthogonal, one in the  CAC-CAN  plane: */
VectorProduct_ (&u1, &CAN, &CAC);
VectorProduct_ (&u2, &u1, &CAC);

/* Two vectors orthogonal on CAC, mutually */
/* orthogonal, one in  the  CAC-CO  plane: */
VectorProduct_ (&v1, &CAC, &CO);
VectorProduct_ (&v2, &v1, &CAC);

/* The angle alpha, which will be used to calculate phi: */
if ((denom = AbsValue_ (&v2) * AbsValue_ (&u2)) == 0.0)
	{
	printf ("Zero length vector found ...\n");
	return ERRORINDICATOR;
	}
ratio = ScalarProduct_ (&v2, &u2) / denom;
if ((ratio > 1.0) || (ratio < -1.0)) return ERRORINDICATOR;
alpha = acos (ratio);

/* There are two possible solutions; the right one is resolved here: */
if (ScalarProduct_ (&v2, &u1) >= 0) psi = alpha;
else psi = -alpha;

/* Return the angle (in radians): */
return psi;
}

/*===========================================================================*/


