
/*======includes:============================================================*/

#include <stdio.h>
#include "common.h"
#include "protein.h"

/*======function prototypes:=================================================*/

void		ErrorMessage_ (char *, char *, char *,
				char *, char *, char *, char *);
void		WarningMessage_ (char *, char *, char *,
				 char *, char *, char *, char *);

/*======write dih. angles, res. names and numbers and chain ID's to file:====*/

int DihedWriteResults_ (char *fname,
			DihedS *DSP, ResidueS *RSP, long ResiduesN,
			FlagS *FSP)
{
FILE		*fp;
long		ResI;
ResidueS	*rp;
double		phi, psi, ratio;

if (!(fp = fopen (fname, "w")))
	{
	ErrorMessage_ ("dihed", "DihedWriteResults", fname,
			"Failed to open output file!",
			" Check file name and path!\n", "", "");
	return -1;
	}

ratio = 180.0 / PIVALUE;

/* Write data: */
for (ResI = 0L; ResI < ResiduesN; ResI++)
	{
	rp = RSP + ResI;
	phi = (DSP + ResI)->phi;
	psi = (DSP + ResI)->psi;

	/** Omit bad data, if flag -o used: **/
	if ((FSP + 2)->Flag)
		{
		if ((phi == ERRORINDICATOR) ||
		    (psi == ERRORINDICATOR)) continue;
		}

	/** Write data, using natural units (radians): **/
	if ((FSP + 0)->Flag)
		{
		/** If flag -s was used, write the short output: **/
		if ((FSP + 1)->Flag)
			{
			fprintf (fp, " %8.3f %8.3f\n", phi, psi);
			}
		else
			{
			fprintf (fp, " %8.3f %8.3f   %3s %4ld %1s\n", phi, psi,
				 rp->Name, rp->Number, rp->ChainIdentifier);
			}
		}

	/** Write data, using degrees: **/
	else
		{
		if (phi != ERRORINDICATOR) phi *= ratio;
		if (psi != ERRORINDICATOR) psi *= ratio;
		/** If flag -s was used, write the short output: **/
                if ((FSP + 1)->Flag)
			{
			fprintf (fp, " %8.1f %8.1f\n", phi, psi);
			}
		else
			{
			fprintf (fp, " %8.3f %8.3f   %3s %4ld %1s\n", phi, psi,
				 rp->Name, rp->Number, rp->ChainIdentifier);
			}
		}
	}

fclose (fp);

if (ResI <= 1L)
	{
	WarningMessage_ ("dihed", "DihedWriteResults", "",
			"Nothing to write!\n", "", "", "");
	return -1;
	}

return 1;
}

/*===========================================================================*/


