/*=============================================================================

Return values:
	(1) Positive on success.
	(2) Zero if data ignored.
	(3) Negative on failure.

========includes:============================================================*/

#include <stdio.h>
#include <string.h>
#include "common.h"
#include "protein.h"
#include "pdb.h"

/*======function prototypes:=================================================*/

int		ExtractLong_ (long *, char *);
int		ExtractString_ (char *, int, char *, int, int);

/*======extract atomic data (including residue data):========================*/

int ExtractAtomicData_ (AtomDataS *ASP, AtomFormatS *AFSP, char *s)
{
int		MinLen = 66;
static char	PrevAtomName[ATOMNAMESIZE] = "####";
char		AuxString[STRINGSIZE];
long		l;
static long	AltPosN;
double		x, y, z;

/* Check is line too short: */
if (strlen (s) < MinLen) return -1;

/* Extract atom serial number: */
ExtractString_ (AuxString, (int) STRINGSIZE, s,
		AFSP->AtomNumberStart, AFSP->AtomNumberEnd);
if (ExtractLong_ (&l, AuxString) < 0) return -2;
ASP->AtomNumber = l;

/* Extract atom name: */
/** First store the name in auxilliary string: **/
ExtractString_ (AuxString, (int) ATOMNAMESIZE, s,
		AFSP->AtomNameStart, AFSP->AtomNameEnd);
/** Check is this name equal to the previous name (alternate position!): **/
if (strcmp (AuxString, PrevAtomName) == 0)
	{
	if (AltPosN <= MAXMSG)
		{
		printf ("Alternate position found (ignored) ...\n");
		}
	else if (AltPosN == MAXMSG + 1)
		{
		printf ("Alternate position found again,");
		printf (" this is the last warning ...\n");
		}
	return 0;
	}
/** If current atom name is not equal to the previous atom name, store it: **/
strncpy (ASP->AtomName, AuxString, ATOMNAMESIZE - 1);
*(ASP->AtomName + ATOMNAMESIZE - 1) = '\0';
/** Copy the current atom name to the previous atom name: **/
strncpy (PrevAtomName, AuxString, ATOMNAMESIZE - 1);
*(PrevAtomName + ATOMNAMESIZE - 1) = '\0';

/* Extract residue name: */
ExtractString_ (ASP->ResidueName, (int) RESNAMESIZE, s,
		AFSP->ResidueNameStart, AFSP->ResidueNameEnd);

/* Extract residue sequence number: */
ExtractString_ (AuxString, (int) RESNUMBERSIZE, s,
		AFSP->ResidueNumberStart, AFSP->ResidueNumberEnd);
if (ExtractLong_ (&l, AuxString) < 0) return -3;
ASP->ResidueNumber = l;

/* Extract chain ID: */
ExtractString_ (ASP->ChainIdentifier, (int) CHAINIDSIZE, s,
		AFSP->ChainIDStart, AFSP->ChainIDEnd);

/* Extract x coordinate: */
ExtractString_ (AuxString, (int) STRINGSIZE, s, AFSP->xStart, AFSP->xEnd);
if (sscanf (AuxString, "%lf", &x) == 1) ASP->x = x;
else ASP->x = ERRORINDICATOR;

/* Extract y coordinate: */
ExtractString_ (AuxString, (int) STRINGSIZE, s, AFSP->yStart, AFSP->yEnd);
if (sscanf (AuxString, "%lf", &y) == 1) ASP->y = y;
else ASP->x = ERRORINDICATOR;

/* Extract z coordinate: */
ExtractString_ (AuxString, (int) STRINGSIZE, s, AFSP->zStart, AFSP->zEnd);
if (sscanf (AuxString, "%lf", &z) == 1) ASP->z = z;
else ASP->x = ERRORINDICATOR;

/* On success, return value is positive: */
return 1;
}

/*===========================================================================*/


