
/*======includes:============================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "protein.h"
#include "pdb.h"

/*======function prototypes:=================================================*/

void		ErrorMessage_ (char *, char *, char *,
			       char *, char *, char *, char *);

/*======extract residue data from atomic data:===============================*/

ResidueS *ExtractResidueData_ (long *ResiduesNP, AtomDataS *ASP, long AtomsN)
{
ResidueS	*RSP;
long		AtomI;
size_t		StructSize, MemSize, MemLeft;
long		OldNumber = -9999L, NewNumber;

/* Allocate the memory (initial amount, reallocate later if necessary): */
MemSize = (size_t) SMALLCHUNKSIZE;
if (!(RSP = (ResidueS *) malloc (MemSize)))
	{
	ErrorMessage_ ("dihed", "ExtractResidueData", "",
		"Failed to allocate memory.",
		" Check the amount of free memory!\n", "", "");
	return NULL;
	}

/* Initialize other numbers: */
*ResiduesNP = 0L;
StructSize = (size_t) sizeof (ResidueS);
MemLeft = MemSize;

/* Examine the complete set of atomic data: */
for (AtomI = 0L; AtomI < AtomsN; AtomI++)
	{
	/* Compare old and new residue number: */
	NewNumber = (ASP + AtomI)->ResidueNumber;
	if (NewNumber == OldNumber) continue;

	/** Set the old residue number: **/
	OldNumber = NewNumber;

	/* If numbers do not match, a new residue was found: */
	/** Start index of a new residue: **/
	(RSP + *ResiduesNP)->Start = AtomI;

	/** End of a previous residue: **/
	if ((AtomI > 0L) && (*ResiduesNP > 0L))
		{
		(RSP + *ResiduesNP - 1)->End = AtomI - 1;
		}

	/** Copy the residue name: **/
	strncpy ((RSP + *ResiduesNP)->Name,
		 (ASP + AtomI)->ResidueName, RESNAMESIZE - 1);
	*((RSP + *ResiduesNP)->Name + RESNAMESIZE - 1) = '\0';

	/** Copy the residue number: **/
	(RSP + *ResiduesNP)->Number = NewNumber;

	/** Copy the chain ID: **/
	strncpy ((RSP + *ResiduesNP)->ChainIdentifier,
		 (ASP + AtomI)->ChainIdentifier, CHAINIDSIZE - 1);
	*((RSP + *ResiduesNP)->ChainIdentifier + CHAINIDSIZE - 1) = '\0';

	/** Increment the number of residues: **/
	(*ResiduesNP)++;
	MemLeft -= StructSize;

        /* Check is there enough memory; if not, reallocate: */
	if (MemLeft < 2 * StructSize)
		{
		MemSize += (size_t) SMALLCHUNKSIZE;
		MemLeft += (size_t) SMALLCHUNKSIZE;
		if (!(RSP = (ResidueS *) realloc (RSP, MemSize)))
			{
			ErrorMessage_ ("dihed", "ExtractResidueData", "",
				"Failed to reallocate memory - exiting!\n",
				"", "", "");
			free (RSP);
			return NULL;
			}
		}
	}

/* The last atom in the last residue: */
(RSP + *ResiduesNP - 1)->End = AtomI - 1;

return RSP;
}

/*===========================================================================*/


