
/*======includes:============================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"
#include "protein.h"
#include "pdb.h"

/*======function prototypes:=================================================*/

void		ErrorMessage_ (char *, char *, char *,
			       char *, char *, char *, char *);
void		WarningMessage_ (char *, char *, char *,
				 char *, char *, char *, char *);
int		ExtractAtomicData_ (AtomDataS *, AtomFormatS *, char *);

/*======read PDB file, store data in a structure array:======================*/

AtomDataS *ReadPDBFile_ (long *AtomsNP, AtomFormatS *AFSP, char *FileName)
{
FILE		*File;
AtomDataS	*ASP;
size_t		MemLeft, StructSize, MemSize;
char		Line[STRINGSIZE];
int		LineSize;
int		n, WarningsN = 0;

/* Open the input file: */
if (!(File = fopen (FileName, "r")))
	{
	ErrorMessage_ ("dihed", "ReadPDBFile", FileName,
		"Failed to open file.",
		" Check the file name specified!\n", "", "");
	return NULL;
	}

/* Allocate the memory (initial amount, reallocate later if necessary): */
MemSize = (size_t) CHUNKSIZE;
if (!(ASP = (AtomDataS *) malloc (MemSize)))
	{
	ErrorMessage_ ("dihed", "ReadPDBFile", "",
		"Failed to allocate memory.",
		" Check the amount of free memory!\n", "", "");
	return NULL;
	}

/* Initialize other numbers: */
*AtomsNP = 0L;
StructSize = (size_t) sizeof (AtomDataS);
MemLeft = MemSize;

/* Read line by line: */
LineSize = sizeof (Line);
while (fgets (Line, LineSize, File) != NULL)
	{
	/* Skip all lines which do not contain atomic coordinates: */
	if (strstr (Line, AFSP->AtomID) != Line) continue;

	/* Extract atomic data from the line: */
	n = ExtractAtomicData_ (ASP + *AtomsNP, AFSP, Line);
	if (n < 0)
		{
		if (WarningsN <= MAXWARN)
			{
			WarningMessage_ ("dihed", "ReadPDBFile", FileName,
				"Failed to extract atomic data!\n",
				"", "", "");
			printf ("%s\n", Line);
			WarningsN++;
			continue;
			}
	        else if (WarningsN > MAXWARN)
			{
			ErrorMessage_ ("dihed", "ReadPDBFile", FileName,
				"Too many warnings - exiting!\n",
				"", "", "");
			free (ASP);
			return NULL;
			}
		}

	/* Update counts: */
	(*AtomsNP)++;		/* It is very bad idea to use *AtomsNP++ ! */
	MemLeft -= StructSize;

	/* Check is there enough memory; if not, reallocate: */
	if (MemLeft < 2 * StructSize)
		{
		MemSize += (size_t) CHUNKSIZE;
		MemLeft += (size_t) CHUNKSIZE;
		if (!(ASP = (AtomDataS *) realloc (ASP, MemSize)))
			{
			ErrorMessage_ ("dihed", "ReadPDBFile", FileName,
				"Failed to reallocate memory - exiting!\n",
				"", "", "");
			free (ASP);
			return NULL;
			}
		}
	}

/* Close file: */
fclose (File);

return ASP;
}

/*===========================================================================*/


